<?php

/**
 * @abstract ImageResizeHTML (atqImageResize)
 * @version 1.1.2
 * @since 1.1.2 - 05.10.2007
 * @since 1.1.1 - 05.10.2007
 * @since 1.1.0 - 03.10.2007
 * @author Andreas Kummer, atelierQ Kummer
 * @copyright Copyright &copy; 2007, atelierQ Kummer
 */

class atqImageResize {

	var $htmlContent;

	function atqImageResize($content, $maxWidth, $maxHeight, $mask, & $db) {

		$this->thumbDir = "thumbs";
		$this->db = $db;

		if (!is_array($mask)) {
			$this->mask[0] = $mask;
			$this->mask[1] = $mask;
		} else {
			$this->mask = $mask;
		}

		$this->maxWidth = (empty ($maxWidth)) ? (9999) : ((int) $maxWidth);
		$this->maxHeight = (empty ($maxHeight)) ? (9999) : ((int) $maxHeight);

		preg_match_all('(<img.*?>)', $content, $match);

		for ($i = 0; $i < count($match[0]); $i++) {
			$newPath = $this->resize($match[0][$i]);
			if ($newPath != $match[0][$i]) {
				/*
				 * Das Bild wurde skaliert...
				 */
				$content = str_replace($match[0][$i], $newPath, $content);
			}
		}

		$this->htmlContent = $content;
	}

	function getOutput() {

		return $this->htmlContent;
	}

	function resize($imageTag) {

		global $cfgClient, $client;
		
		preg_match_all('(([a-zA-Z]*)="(.*?)")', $imageTag, $match);

		foreach ($match[1] as $key => $value) {
			$position[$value] = $key;
		}

		if (!isset ($position['src']) || empty ($match[2][$position['src']])) {
			/*
			 * Abbruch, weil keine Bildquelle bezeichnet ist.
			 */
			return $imageTag;
		}

		$width = min($this->maxWidth, (isset ($position['width'])) ? ($match[2][$position['width']]) : (99999));
		$height = min($this->maxHeight, (isset ($position['height'])) ? ($match[2][$position['height']]) : (99999));

		$imageSourceHtml = $cfgClient[$client]['path']['htmlpath'] . $match[2][$position['src']];
		if (substr($match[2][$position['src']], 0, 4) != 'http') {
			$imageSource = $cfgClient[$client]['path']['frontend'] . $match[2][$position['src']];
		} else {
			$imageSource = $cfgClient[$client]['path']['frontend'] . str_replace($cfgClient[$client]['path']['htmlpath'], '', $match[2][$position['src']]);
		}
		$targetName = str_replace('/', '_', $match[2][$position['src']]) . '.' . $width . 'x' . $height . '.jpg';

		if (!file_exists($imageSource)) {
			/*
			 * Wenn die Originaldatei nicht vorhanden ist, ist abzubrechen.
			 */
			return $imageTag;
		}

		$imageSourceDimensions = getimagesize($imageSource);
		$origImageSourceDimensions = $imageSourceDimensions;

		if (file_exists($cfgClient[$client]['upl']['path'] . $this->thumbDir . '/' . $targetName)) {
			/*
			 * Skaliertes Bild liegt bereits vor und wird zurückgegeben.
			 */
			$imageSourceDimensions = getimagesize($cfgClient[$client]['upl']['path'] . $this->thumbDir . '/' . $targetName);
			if (isset ($position['width'])) {
				$match[0][$position['width']] = 'width="' . $imageSourceDimensions[0] . '"';
			}
			if (isset ($position['height'])) {
				$match[0][$position['height']] = 'height="' . $imageSourceDimensions[1] . '"';
			}
			$match[0][$position['src']] = 'src="' . $cfgClient[$client]['upl']['htmlpath'] . $this->thumbDir . '/' . $targetName . '"';

			$parts['src'] = $cfgClient[$client]['upl']['htmlpath'] . $this->thumbDir . '/' . $targetName;
			$parts['height'] = $imageSourceDimensions[1];
			$parts['width'] = $imageSourceDimensions[0];
			$parts['origHeight'] = $origImageSourceDimensions[1];
			$parts['origWidth'] = $origImageSourceDimensions[0];
			$parts['title'] = (isset ($position['title'])) ? ($match[2][$position['title']]) : ('');
			$parts['titleUrl'] = urlencode($parts['title']);
			$parts['alt'] = (isset ($position['alt'])) ? ($match[2][$position['alt']]) : ('');

			return $this->getFilledMask($parts, $imageSourceHtml, $this->mask[1], $match);
		}

		if ($imageSourceDimensions[0] <= $width && $imageSourceDimensions[1] <= $height) {
			/*
			 * Das Bild muss nicht skaliert werden.
			 */
			$parts['src'] = $match[2][$position['src']];
			$parts['height'] = $imageSourceDimensions[1];
			$parts['width'] = $imageSourceDimensions[0];
			$parts['origHeight'] = $imageSourceDimensions[1];
			$parts['origWidth'] = $imageSourceDimensions[0];
			$parts['title'] = (isset ($position['title'])) ? ($match[2][$position['title']]) : ('');
			$parts['titleUrl'] = urlencode($parts['title']);
			$parts['alt'] = (isset ($position['alt'])) ? ($match[2][$position['alt']]) : ('');

			return $this->getFilledMask($parts, $imageSourceHtml, $this->mask[0], $match);
		}

		/*
		 * Zielgrösse des Bildes ermitteln.
		 */
		$slopeSource = $imageSourceDimensions[1] / $imageSourceDimensions[0];
		$slopeTarget = $height / $width;
		if ($slopeSource < $slopeTarget) {
			/*
			 * Auf Breite skalieren
			 */
			$targetWidth = (int) $width;
			$targetHeight = (int) round(($targetWidth / $imageSourceDimensions[0]) * $imageSourceDimensions[1]);
		} else {
			/*
			 * Auf Höhe skalieren
			 */
			$targetHeight = (int) $height;
			$targetWidth = (int) round(($targetHeight / $imageSourceDimensions[1]) * $imageSourceDimensions[0]);
		}

		$sourcePath = $dst_im = imagecreatetruecolor($targetWidth, $targetHeight);
		$weiss = ImageColorAllocate($dst_im, 255, 255, 255);
		imagefill($dst_im, 0, 0, $weiss);

		if ($imageSourceDimensions[2] == 1) {
			$src_im = imagecreatefromGIF($imageSource);
		}
		elseif ($imageSourceDimensions[2] == 2) {
			$src_im = ImageCreateFromJPEG($imageSource);
		}
		elseif ($imageSourceDimensions[2] == 3) {
			$src_im = ImageCreateFromPNG($imageSource);
		} else {
			$src_im = imagecreatefromgd($imageSource);
		}

		imagecopyresampled($dst_im, $src_im, 0, 0, 0, 0, $targetWidth, $targetHeight, $imageSourceDimensions[0], $imageSourceDimensions[1]);
		imagejpeg($dst_im, $cfgClient[$client]['upl']['path'] . $this->thumbDir . '/' . $targetName, 100);

		if (isset ($position['width'])) {
			$match[0][$position['width']] = 'width="' . $targetWidth . '"';
		}
		if (isset ($position['height'])) {
			$match[0][$position['height']] = 'height="' . $targetHeight . '"';
		}
		$match[0][$position['src']] = 'src="' . $cfgClient[$client]['upl']['htmlpath'] . $this->thumbDir . '/' . $targetName . '"';

		$parts['src'] = $cfgClient[$client]['upl']['htmlpath'] . $this->thumbDir . '/' . $targetName;
		$parts['height'] = $targetHeight;
		$parts['width'] = $targetWidth;
		$parts['origHeight'] = $imageSourceDimensions[1];
		$parts['origWidth'] = $imageSourceDimensions[0];
		$parts['title'] = (isset ($position['title'])) ? ($match[2][$position['title']]) : ('');
		$parts['titleUrl'] = urlencode($parts['title']);
		$parts['alt'] = (isset ($position['alt'])) ? ($match[2][$position['alt']]) : ('');

		return $this->getFilledMask($parts, $imageSourceHtml, $this->mask[1], $match);
	}

	function getFilledMask($parts, $imageSourceHtml, $mask, $match) {

		global $cfgClient, $client;
		
		$returnValue = $mask;

		/*
		 * Alle übertragenen Platzhalter ersetzen.
		 */
		foreach ($parts as $key => $value) {
			$returnValue = str_replace('{' . $key . '}', $value, $returnValue);
		}
		
		/*
		 * Description aus der Datenbank auslesen und in der Maske ersetzen.
		 */
		$this->db->query("" .
				"SELECT description " .
				"FROM con_upl " .
				"WHERE CONCAT('{$cfgClient[$client]['upload']}', dirname, filename) = '{$imageSourceHtml}' " .
				"");
				
		if ($this->db->next_record()) {
			$returnValue = str_replace('{description}', urldecode($this->db->f('description')), $returnValue);
		} else {
			$returnValue = str_replace('{description}', '', $returnValue);
		}
		
		$returnValue = str_replace('{origSrc}', $imageSourceHtml, $returnValue);
		
		/*
		 * Alle im Image-Tag allenfalls sonst noch vorhandenen Tags auslesen und - sofern
		 * in der Maske vorhanden - ersetzen.
		 */
		for ($i = 0; $i < count($match[1]); $i++) {
			$returnValue = str_replace('{' . $match[1][$i] . '}', $match[2][$i], $returnValue);
		}
		
		/*
		 * Allenfalls immer noch vorhandene Platzhalter, die nicht haben ersetzt werden können, sind zu löschen.
		 */
		$returnValue = preg_replace('/{.*?}/', '', $returnValue);

		return $returnValue;
	}

}
?>
